use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new(
   module_name => 'Object::Pad',
   requires => {
      'indirect' => 0,

      'perl' => '5.016', # pad_add_name_pvn, pad_add_name_pvs, gv_init_pvn
   },
   test_requires => {
      'Data::Dump' => 0,
      'Test::More' => '0.88', # done_testing
   },
   configure_requires => {
      'Module::Build' => '0.4004', # test_requires
   },
   license => 'perl',
   create_license => 1,
   create_readme  => 1,

   extra_compiler_flags => [qw( -Ihax )],
);

$build->create_build_script;
